@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2021, Tomas Babej, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -26,6 +26,7 @@
 
 #include <cmake.h>
 #include <CmdCustom.h>
+#include <random>
 #include <sstream>
 #include <map>
 #include <vector>
@@ -59,6 +60,21 @@ CmdCustom::CmdCustom (
   _category              = Category::report;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// Whether a report uses context is defined by the report.<name>.context
+// configuration variable.
+//
+bool CmdCustom::uses_context () const
+{
+  auto config = Context::getContext ().config;
+  auto key = "report." + _keyword + ".context";
+
+  if (config.has (key))
+    return config.getBoolean (key);
+  else
+    return _uses_context;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 int CmdCustom::execute (std::string& output)
 {
@@ -87,8 +103,11 @@ int CmdCustom::execute (std::string& output)
   if (reportFilter != "")
     Context::getContext ().cli2.addFilter (reportFilter);
 
-  // Apply filter.
+  // Make sure reccurent tasks are generated.
+  handleUntil ();
   handleRecurrence ();
+
+  // Apply filter.
   Filter filter;
   std::vector <Task> filtered;
   filter.subset (filtered);
@@ -230,6 +249,29 @@ int CmdCustom::execute (std::string& output)
     rc = 1;
   }
 
+  // Inform user about the new release highlights if not presented yet
+  if (Context::getContext ().config.get ("news.version") != "2.6.0")
+  {
+    std::random_device device;
+    std::mt19937 random_generator(device());
+    std::uniform_int_distribution<std::mt19937::result_type> twentyfive_percent(1, 4);
+
+    std::string NEWS_NOTICE = (
+      "Recently upgraded to 2.6.0. "
+      "Please run 'task news' to read highlights about the new release."
+    );
+
+    // 1 in 10 chance to display the message.
+    if (twentyfive_percent(random_generator) == 4)
+    {
+      if (Context::getContext ().verbose ("footnote"))
+        Context::getContext ().footnote (NEWS_NOTICE);
+      else if (Context::getContext ().verbose ("header"))
+        Context::getContext ().header (NEWS_NOTICE);
+    }
+  }
+
+
   feedback_backlog ();
   output = out.str ();
   return rc;
